/****************************************************************************

    TwoDPanelDlg Class
    Copyright (C) 2008-2019 Andre Deperrois

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/


#ifndef TWODPANELDLG_H
#define TWODPANELDLG_H

#include <QDialog>
#include <QDialogButtonBox>
#include <QPushButton>


class IntEdit;
class DoubleEdit;
class Foil;
class XFoil;

class TwoDPanelDlg : public QDialog
{
    Q_OBJECT
    friend class AFoil;
    friend class XDirect;

private slots:
    void onApply();
    void onOK();
    void onChanged();
    void onButton(QAbstractButton *pButton);

public:
    TwoDPanelDlg(QWidget *pParent);

    static XFoil *s_pXFoil;

    void initDialog();

private:
    void keyPressEvent(QKeyEvent *event);
    void setupLayout();
    void readParams();

    QDialogButtonBox *m_pButtonBox;

    IntEdit  *m_pctrlNPanels;
    DoubleEdit *m_pctrlCVpar,  *m_pctrlCTErat, *m_pctrlCTRrat;
    DoubleEdit *m_pctrlXsRef1, *m_pctrlXsRef2, *m_pctrlXpRef1, *m_pctrlXpRef2;

    bool m_bApplied;
    bool m_bModified;

    int npan;
    double cvpar;
    double cterat;
    double ctrrat;
    double xsref1;
    double xsref2;
    double xpref1;
    double xpref2;

    Foil *m_pBufferFoil;
    Foil *m_pMemFoil;

    QWidget *m_pParent;
};

#endif // TWODPANELDLG_H

